#include "YoggieEngine.h"
#include "glfwWindow.h"
#include <chrono>
#include<GLFW/glfw3.h>
namespace YoggieEngine {

	void LoadGLExtensions() {
		if (!gladLoadGLLoader((GLADloadproc)glfwGetProcAddress)) {
			spdlog::error("Failed to initialize GLAD!\n");
			exit(-1);
		}
	}


	static void error_callback(int error, const char* description) {
		spdlog::error("{0}", description);
	}

	glfwWindow::glfwWindow(const int width, const int height, const char* title)
	: NativeWindow()
	{
		spdlog::debug("Creating window");
		m_width = width;
		m_height = height;
		m_fullscreen = false;


		glfwSetErrorCallback(error_callback);

		auto start = std::chrono::high_resolution_clock::now();
		if (!glfwInit()) {

			spdlog::error("Failed to initialise GLFW!");
			exit(-1);
		}
		auto end =  std::chrono::high_resolution_clock::now();
		auto duration = std::chrono::duration_cast<std::chrono::milliseconds> (end - start);
		spdlog::info("GLFWInit() call took {0} milliseconds.", duration.count());

		glfwWindowHint(GLFW_DOUBLEBUFFER, GLFW_TRUE);
		glfwWindowHint(GLFW_FOCUS_ON_SHOW, GLFW_TRUE);
		if (m_fullscreen) {
			glfwWindowHint(GLFW_MAXIMIZED, GLFW_TRUE);
		}

		if (!m_resizable) {
			glfwWindowHint(GLFW_RESIZABLE, GLFW_TRUE);
		}
	

		window = glfwCreateWindow(m_width, m_height, title, NULL, NULL);

		if (!window) {
			spdlog::error("GLFW failed to create a window!");
			glfwTerminate();
			exit(-1);
		}

		SetContext();
	

		if (!m_vsync) {
			glfwSwapInterval(0);
		}

		glfwGetFramebufferSize(window, &m_width, &m_height);
		LoadGLExtensions();


	}

	glfwWindow::~glfwWindow()
	{
		glfwTerminate();
	}

	bool glfwWindow::WindowShouldClose()
	{
		return glfwWindowShouldClose(window);
	}

	void glfwWindow::Poll()
	{
		glfwPollEvents();
	}

	void glfwWindow::SwapBuffers()
	{
		glfwSwapBuffers(window);
	}

	void* glfwWindow::GetHandle()
	{
		return window;
	}

	void glfwWindow::SetViewPort(int width, int height)
	{
		glViewport(0, 0, width, height);
	}

	void glfwWindow::SetContext()
	{
		glfwMakeContextCurrent(window);
	}
}

